

## ------------------------------------------------------------------------ ##
##  R scripts for " Multidimenional smoothing by adaptive local kernel      ##
##                weighted log-likelihood: Application to long-term care    ##
##                insurance "                                               ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for extracting original data  - Script n•01       ##
## ------------------------------------------------------------------------ ##
##  Description   Script to load, create, save and plot the original data   ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 30/08/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...") # ! to be added !

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ------------------------------------------------------------------------ ##
##             	                  Raw data                                  ##
## ------------------------------------------------------------------------ ##

## ------------------ Set age of occurence and duration (months)

Occ <- 70:90; Dur <- 0:119

## ------------------ Load .csv files

E_i.j <- read.table("Data/Original data/... .csv",header=TRUE,sep=";", colClasses = "character")
for (i in 1:ncol(E_i.j)) { E_i.j[i] <- as.numeric(sub(",",".",E_i.j[,i])) }
E_i.j <- as.matrix(E_i.j[1:120,]); colnames(E_i.j) <- as.character(Occ)

Q_i.j <- read.table("Data/Original data/... .csv",header=TRUE,sep=";", colClasses = "character")
for (i in 1:ncol(Q_i.j)) { Q_i.j[i] <- as.numeric(sub(",",".",Q_i.j[,i])) }
Q_i.j <- as.matrix(Q_i.j); colnames(Q_i.j) <- as.character(Occ)

## ------------------ Central death rate

Mu_i.j <- (2*Q_i.j)/(2-Q_i.j)

## ------------------ Number of deaths

D_i.j <- Mu_i.j * E_i.j

## ------------------ Final surface from technical report

Qhat_i.j <- read.table("Data/Original data/... .csv",header=TRUE,sep=";", colClasses = "character")
for (i in 1:ncol(Qhat_i.j)) { Qhat_i.j[i] <- as.numeric(sub(",",".", Qhat_i.j[,i])) }
Qhat_i.j <- as.matrix(Qhat_i.j[1:120, 1:21] ); colnames(Qhat_i.j) <- as.character(Occ)

## ------------------------------------------------------------------------ ##
##             	                 Surface plot                               ##
## ------------------------------------------------------------------------ ##

## ------------------ Number of deaths

x11();
surface_plot(D_i.j, expression(paste(D[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Exposition

x11();
surface_plot(E_i.j, expression(paste(E[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Mortality rate

x11();
surface_plot(Q_i.j, expression(paste(q[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Forces of mortality / central death rate

## --- Original scale

x11();
surface_plot(Mu_i.j, expression(paste(mu[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## --- Log scale

x11();
surface_plot(log(Mu_i.j), expression(paste("log ",mu[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Final mortality rate from technical report

## --- Original scale

x11();
surface_plot(Qhat_i.j, expression(paste(widehat(q)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## --- Log scale

x11();
surface_plot(log(Qhat_i.j), expression(paste("log ",widehat(q)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------------------------------------------------------------ ##
##             	                  Save data                                 ##
## ------------------------------------------------------------------------ ##

save(D_i.j, file = "Data/Original data/dij.RData")
save(E_i.j, file = "Data/Original data/eij.RData")
save(Q_i.j, file = "Data/Original data/qij.RData")
save(Mu_i.j, file = "Data/Original data/muij.RData")
save(Qhat_i.j, file = "Data/Original data/qhatij.RData")

