

## ------------------------------------------------------------------------ ##
##  R scripts for " Multidimenional smoothing by adaptive local kernel      ##
##                weighted log-likelihood: Application to long-term care    ##
##                insurance "                                               ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for smoothing mu_i.j  - Script n•02_1             ##
## ------------------------------------------------------------------------ ##
##  Description   Script to smooth the forces of mortality by local log		##
##                Poisson likelihood                                        ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 30/08/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...") # ! to be added !

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ---------------- LOCFIT (Use locfit functions to speed up the computations)

require(locfit)

## ------------------------------------------------------------------------ ##
##             	                  Load data                                 ##
## ------------------------------------------------------------------------ ##

## ------------------ Set age of occurence and duration (months)

Occ <- 70:90; Dur <- 0:119

## ------------------ Load data

load("Data/Original data/dij.RData")
load("Data/Original data/eij.RData")
load("Data/Original data/muij.RData")

## ------------------------------------------------------------------------ ##
##                        local likelihood smoothing                        ##
## ------------------------------------------------------------------------ ##

## ---------------- Optimal Constellation of Smoothing Parameters
## ---------------- Mu_i.j ~ Poisson(link=log) with weights = E_i.j

## --- For a set of parameters,

hh <- 1 : 20; pp <- 0 : 3; k <- c("epan"); fam <- "poisson"; lk <- "log"; ww <- c(E_i.j)

## --- Compute criteria and Fitted DF

m.crit_lk <- get.crit.lk(D_i.j, Occ, Dur, hh, pp, k, fam, lk, ww)

#save(m.crit_lk, file = "Data/Smoothed data/m.crit_lk.RData")

#load("Data/Smoothed data/m.crit_lk_lk.RData")

## - Plot the criteria vs Fitted DF

plot.crit("AIC", m.crit_lk$AICMAT, m.crit_lk$V2MAT, c(2300,2510), c(10,100))
plot.crit("LCV", m.crit_lk$LCVMAT, m.crit_lk$V2MAT, c(2300,2360), c(10,100))

## - Keep h = 13 & p = 2 with AIC criterion // 

m.crit_lk$AICMAT[13,3]; m.crit_lk$V2MAT[13,3]

plot.crit("AIC", m.crit_lk$AICMAT, m.crit_lk$V2MAT, c(2300,2510), c(10,100))
trellis.focus("panel", 1, 1, highlight = FALSE)
panel.points(m.crit_lk$V2MAT[13,3], m.crit_lk$AICMAT[13,3], pch=16, col=gray(.9), cex=3, lwd = 2)
panel.points(m.crit_lk$V2MAT[13,3], m.crit_lk$AICMAT[13,3], pch="2", col=1, cex=1.5)
trellis.unfocus()

## --- Compute the fit for h = 13

nber.p <- get.enp.target(13); nber.p
span <- nber.p / (length(Dur)*length(Occ)); span

mod_lk <- locfit(c(as.matrix(D_i.j)) ~ lp(c(rep(Dur,length(Occ))),c(rep(Occ,each= length(Dur))),deg = 2, nn = span,  scale=1),ev=dat(), family = fam, link = lk, kern = k, weights = ww)

fff <- predict(mod_lk)

Mu_i.j_hat_lk <- matrix(fff,length(Dur),length(Occ))

## ------------------ Surface plot

## --- Original scale

x11();
surface_plot(Mu_i.j_hat_lk, expression(paste(widehat(mu)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## --- Log scale

x11();
surface_plot(log(Mu_i.j_hat_lk), expression(paste("log ",widehat(mu)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Save fit

save(Mu_i.j_hat_lk, file = "Data/Smoothed data/Mu_i.j_hat_lk.RData")
save(mod_lk, file = "Data/Smoothed data/mod_lk.RData")


