

## ------------------------------------------------------------------------ ##
##  R scripts for " Multidimenional smoothing by adaptive local kernel      ##
##                weighted log-likelihood: Application to long-term care    ##
##                insurance "                                               ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for smoothing mu_i.j  - Script n•02_2             ##
## ------------------------------------------------------------------------ ##
##  Description   Script to smooth the forces of mortality by penalized     ##
##                B-splines                                                 ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 30/08/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...") # ! to be added !

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## --- Use functions in MortalitySmooth package

require(MortalitySmooth)

## ------------------------------------------------------------------------ ##
##             	                  Load data                                 ##
## ------------------------------------------------------------------------ ##

## ------------------ Set age of occurence and duration (months)

Occ <- 70:90; Dur <- 0:119

## ------------------ Load data

load("Data/Original data/dij.RData")
load("Data/Original data/eij.RData")
load("Data/Original data/muij.RData")

## ------------------------------------------------------------------------ ##
##                       Penalized B-splines smoothing                      ##
## ------------------------------------------------------------------------ ##

## ---------------- Optimal number of knots

y <- D_i.j
os <- E_i.j

## --- Mimimize BIC, (Have to select method = 1)
## --- Mimimize AIC, (Have to select method = 2)
	
mod_ps <- Mort2Dsmooth(x=Dur, y=Occ, Z=y, offset=log(os+(os==0)), deg=c(3,3), pord=c(2,2), method=1)

fff <- predict(mod_ps)

Mu_i.j_hat_ps <- matrix(exp(fff),length(Dur),length(Occ))

colnames(Mu_i.j_hat_ps) <- as.character(Occ)

mod_ps$df; mod_ps$bic; mod_ps$ndx
# mod_ps$Bx; mod_ps$By
mod_ps$lambdas

## ------------------ Surface plot

## --- Original scale

x11();
surface_plot(Mu_i.j_hat_ps, expression(paste(widehat(mu)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## --- Log scale

x11();
surface_plot(log(Mu_i.j_hat_ps), expression(paste("log ",widehat(mu)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Save fit

save(Mu_i.j_hat_ps, file = "Data/Smoothed data/Mu_i.j_hat_ps.RData")
save(mod_ps, file = "Data/Smoothed data/mod_ps.RData")

