

## ------------------------------------------------------------------------ ##
##  R scripts for " Multidimenional smoothing by adaptive local kernel      ##
##                weighted log-likelihood: Application to long-term care    ##
##                insurance "                                               ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for smoothing mu_i.j  - Script n•02_3             ##
## ------------------------------------------------------------------------ ##
##  Description   Script to smooth the forces of mortality by adaptive      ##
##                local log Poisson likelihood - Pointwise mtehod - ICI     ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 30/08/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...") # ! to be added !

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ---------------- LOCFIT (Use locfit functions to speed up the computations)

require(locfit)

## ------------------------------------------------------------------------ ##
##             	                  Load data                                 ##
## ------------------------------------------------------------------------ ##

## ------------------ Set age of occurence and duration (months)

Occ <- 70:90; Dur <- 0:119

## ------------------ Load data

load("Data/Original data/dij.RData")
load("Data/Original data/eij.RData")
load("Data/Original data/muij.RData")

## ------------------------------------------------------------------------ ##
##      Adaptive local log Poisson likelihood - Pointwise method - ICI      ##
## ------------------------------------------------------------------------ ##

## ------------------ Load parameters

k <- c("epan"); fam <- "poisson"; lk <- "log"; ww <- c(E_i.j)

## ------------------ Fit model

mod_ici <- locfit(c(as.matrix(D_i.j)) ~ c(rep(Dur,length(Occ)))*c(rep(Occ,each= length(Dur))),deg = 2, alpha = c(0,45,.1), acri="ici", family = "poisson", link = "log", kern = "gauss", ev=dat(), scale=FALSE, weights = c(E_i.j))

## ------------------ Extract the bandwidth

a_h <- matrix(sqrt((predict(mod_ici,where="fitp",what="band")/pi)),length(Dur),length(Occ))

x11();
surface_plot(a_h, "Adaptive Bandwidth", c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Obtain the fit

fff <- predict(mod_ici)

Mu_i.j_hat_ici <- matrix(fff,length(Dur),length(Occ))

colnames(Mu_i.j_hat_ici) <- as.character(Occ)

## ------------------ Surface plot

## --- Original scale

x11();
surface_plot(Mu_i.j_hat_ici, expression(paste(widehat(mu)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## --- Log scale

x11();
surface_plot(log(Mu_i.j_hat_ici), expression(paste("log ",widehat(mu)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Save fit and model

save(Mu_i.j_hat_ici, file = "Data/Smoothed data/Mu_i.j_hat_ici.RData")
save(mod_ici, file = "Data/Smoothed data/mod_ici.RData")
