

## ------------------------------------------------------------------------ ##
##  R scripts for " Multidimenional smoothing by adaptive local kernel      ##
##                weighted log-likelihood: Application to long-term care    ##
##                insurance "                                               ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for smoothing mu_i.j  - Script n•02_4             ##
## ------------------------------------------------------------------------ ##
##  Description   Script to smooth the forces of mortality by adaptive      ##
##                local log Poisson likelihood - Bandiwth factor            ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 30/08/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...") # ! to be added !

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ---------------- LOCFIT (Use locfit functions to speed up the computations)

require(locfit)

## ------------------------------------------------------------------------ ##
##             	                  Load data                                 ##
## ------------------------------------------------------------------------ ##

## ------------------ Set age of occurence and duration (months)

Occ <- 70:90; Dur <- 0:119

## ------------------ Load data

load("Data/Original data/dij.RData")
load("Data/Original data/eij.RData")
load("Data/Original data/muij.RData")

## ------------------------------------------------------------------------ ##
##      Adaptive local log Poisson likelihood - Bandwidth factors method    ##
## ------------------------------------------------------------------------ ##

## ------------------ Load parameters

k <- c("epan"); fam <- "poisson"; lk <- "log"; ww <- c(E_i.j)

y <- D_i.j
x1 <- matrix(Dur, length(Dur), length(Occ))
x2 <- matrix(Occ, length(Dur), length(Occ), byrow = T)
os <- E_i.j

mod_bf <- loc.lk.h.dep(y, os, x1, x2, 70, 2, "epan", 0.15, os)

x11();
surface_plot(mod_bf$l_i, "Bandwidth factors", c(min(Dur),max(Dur),min(Occ),max(Occ)))

x11();
surface_plot(mod_bf$h.dep, "Bandwidth (radius)", c(min(Dur),max(Dur),min(Occ),max(Occ)))


x11();
surface_plot(mod_bf$InflMAT, "Influence values", c(min(Dur),max(Dur),min(Occ),max(Occ)))

Mu_i.j_hat_bf <- mod_bf$muxhat

colnames(Mu_i.j_hat_bf) <- as.character(Occ)

## ------------------ Surface plot

## --- Original scale

x11();
surface_plot(Mu_i.j_hat_bf, expression(paste(widehat(mu)[ij])), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------ Save

save(Mu_i.j_hat_bf, file = "Data/Smoothed data/Mu_i.j_hat_bf.RData")
save(mod_bf, file = "Data/Smoothed data/mod_bf.RData")

