

## ------------------------------------------------------------------------ ##
##  R scripts for " Multidimenional smoothing by adaptive local kernel      ##
##                weighted log-likelihood: Application to long-term care    ##
##                insurance "                                               ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for obtain the residuals  - Script n•03           ##
## ------------------------------------------------------------------------ ##
##  Description   Script to obtain the response, Pearson and deviance       ##
##                residuals for all the models                              ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 30/08/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...") # ! to be added !

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ---------------- LOCFIT (Use locfit functions to speed up the computations)

require(locfit)

## ------------------------------------------------------------------------ ##
##             	                  Load data                                 ##
## ------------------------------------------------------------------------ ##

## ------------------ Set age of occurence and duration (months)

Occ <- 70:90; Dur <- 0:119

## ------------------ Load data

load("Data/Original data/dij.RData")
load("Data/Original data/eij.RData")
load("Data/Original data/muij.RData")

## ------------------ Load fitted values

load("Data/Smoothed data/Mu_i.j_hat_lk.RData")
load("Data/Smoothed data/Mu_i.j_hat_ps.RData")
load("Data/Smoothed data/Mu_i.j_hat_ici.RData")
load("Data/Smoothed data/Mu_i.j_hat_bf.RData")
load("Data/Original data/qhatij.RData")
Mu_i.j_hat_tech <- (2*Qhat_i.j)/(2-Qhat_i.j)

Mu_i.j_list <- vector("list",5)

Mu_i.j_list[[1]] <- Mu_i.j_hat_lk
Mu_i.j_list[[2]] <- Mu_i.j_hat_ps
Mu_i.j_list[[3]] <- Mu_i.j_hat_ici
Mu_i.j_list[[4]] <- Mu_i.j_hat_bf
Mu_i.j_list[[5]] <- Mu_i.j_hat_tech

for (i in 1:5){ colnames(Mu_i.j_list[[i]]) <- as.character(Occ) }

## ------------------------------------------------------------------------ ##
##             	            Obtain the deviance                             ##
## ------------------------------------------------------------------------ ##

## ------------------ Obtain the single deviance

dev_list <- vector("list",5)

for (i in 1:5){
	dev_list[[i]] <- matrix(,length(Dur),length(Occ))
	colnames(dev_list[[i]]) <- as.character(Occ)
	dev_list[[i]][(D_i.j>0)] <- 2 * (D_i.j[(D_i.j>0)] * log(D_i.j[(D_i.j>0)] /(Mu_i.j_list[[i]]*E_i.j)[(D_i.j>0)])-(D_i.j[(D_i.j>0)]-(Mu_i.j_list[[i]]*E_i.j)[(D_i.j>0)]) )
	dev_list[[i]][(D_i.j==0)] <- 2 * ((Mu_i.j_list[[i]]*E_i.j))[(D_i.j==0)] }
	
#save(dev_list, file="Data/Smoothed data/dev_list.RData")

## ------------------ Compare with the deviance obtained from the models

load("Data/Smoothed data/mod_lk.RData")
load("Data/Smoothed data/mod_ps.RData")
load("Data/Smoothed data/mod_ici.RData")
load("Data/Smoothed data/mod_bf.RData")

dev.mod <- vector(,5)
dev.mod[1] <- - 2 * mod_lk$dp["lk"]
dev.mod[2] <- mod_ps$dev
dev.mod[3] <- - 2 * mod_ici$dp["lk"]
dev.mod[4] <- NA
dev.mod[5] <- NA

for (i in 1:5){ print(c(sum(dev_list[[i]]),dev.mod[i]) ) }

## ------------------------------------------------------------------------ ##
##             	            Obtain the residuals                            ##
## ------------------------------------------------------------------------ ##

resp.res <- pear.res <- dev.res <- vector("list",5)

for (i in 1:5){
	resp.res[[i]] <- Mu_i.j - Mu_i.j_list[[i]]
	pear.res[[i]] <- (D_i.j - Mu_i.j_list[[i]]*E_i.j) / sqrt(Mu_i.j_list[[i]]*E_i.j)
	dev.res[[i]] <- sign(Mu_i.j - Mu_i.j_list[[i]]) * sqrt(abs(dev_list[[i]]))
	colnames(resp.res[[i]]) <- colnames(pear.res[[i]]) <- colnames(dev.res[[i]]) <- as.character(Occ) }
	
#save(resp.res , file="Data/Smoothed data/resp.res.RData")
#save(pear.res , file="Data/Smoothed data/pear.res.RData")
#save(dev.res , file="Data/Smoothed data/dev.res.RData")
	
## ------------------------------------------------------------------------ ##
##             	             Plot the residuals                             ##
## ------------------------------------------------------------------------ ##

names.models <- c("Local lik.", "P-splines", "Local lik. ICI", "Local lik. BF", "Tech. report")

## --- Choose an age of occurence

aa <- 70

x11();
plot.res.yy(resp.res, pear.res, dev.res, aa, Dur, names.models, c("Resp. residuals", "Pear. residuals", "Dev. residuals"))

## --- Surface plot

## - Response residuals

for (i in 1:5){
	x11();
	print(surface_plot(resp.res[[i]], paste("Resp. residuals ",names.models[i]), c(min(Dur),max(Dur),min(Occ),max(Occ)))) }

## - Pearson residuals

for (i in 1:5){
	x11();
	print(surface_plot(pear.res[[i]], paste("Pear. residuals ",names.models[i]), c(min(Dur),max(Dur),min(Occ),max(Occ)))) }

## - Deviance residuals

for (i in 1:5){
	x11();
	print(surface_plot(dev.res[[i]], paste("Dev. residuals ",names.models[i]), c(min(Dur),max(Dur),min(Occ),max(Occ)))) }



