
## ------------------------------------------------------------------------ ##
##  R scripts for "WP4: Adaptive approaches applied to the dependent lives" ##
## ------------------------------------------------------------------------ ##
##  Script        Scripts for obtain the residuals  - Script n•05           ##
## ------------------------------------------------------------------------ ##
##  Description   Script to obtain compare graphicaly the fits and apply    ##
##                tests to compare the smoothing                            ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien                                              ##
## ------------------------------------------------------------------------ ##
##  Version       01 - 30/08/2012                                           ##
## ------------------------------------------------------------------------ ##

## ---------------- If necessary,

rm(list=ls(all=TRUE))

## ------------------------------------------------------------------------ ##
##           	       Change the workspace repository                      ##
## ------------------------------------------------------------------------ ##

setwd("...") # ! to be added !

## ------------------------------------------------------------------------ ##
##           	               Load functions                               ##
## ------------------------------------------------------------------------ ##

## --- Miscellaneous functions

source("misc_functions.R")

## --- Graphic functions

require(lattice); require(grid); require(latticeExtra)

## --- Load X11 options

X11.options(type="nbcairo",family="FreeSans")

## ------------------------------------------------------------------------ ##
##             	                  Load data                                 ##
## ------------------------------------------------------------------------ ##

## ------------------ Set age of occurence and duration (months)

Occ <- 70:90; Dur <- 0:119

## ------------------ Load data

load("Data/Original data/dij.RData")
load("Data/Original data/eij.RData")
load("Data/Original data/muij.RData")
load("Data/Original data/qhatij.RData")

Mu_i.j_hat_tech <- (2*Qhat_i.j)/(2-Qhat_i.j)

## ------------------ Load fitted values

load("Data/Smoothed data/Mu_i.j_hat_lk.RData")
load("Data/Smoothed data/Mu_i.j_hat_ps.RData")
load("Data/Smoothed data/Mu_i.j_hat_ici.RData")
load("Data/Smoothed data/Mu_i.j_hat_bf.RData")

Mu_i.j_list <- vector("list",5)

Mu_i.j_list[[1]] <- Mu_i.j_hat_lk
Mu_i.j_list[[2]] <- Mu_i.j_hat_ps
Mu_i.j_list[[3]] <- Mu_i.j_hat_ici
Mu_i.j_list[[4]] <- Mu_i.j_hat_bf
Mu_i.j_list[[5]] <- Mu_i.j_hat_tech

for (i in 1:5){ colnames(Mu_i.j_list[[i]]) <- as.character(Occ) }

## ------------------ Load models

names.models <- c("Local lik.", "P-splines", "Local lik. ICI", "Loc. lik. Band. Fact.")

load("Data/Smoothed data/mod_lk.RData")
load("Data/Smoothed data/mod_ps.RData")
load("Data/Smoothed data/mod_ici.RData")
load("Data/Smoothed data/mod_bf.RData")

mod_list <- vector("list",4)

mod_list[[1]] <- mod_lk
mod_list[[2]] <- mod_ps
mod_list[[3]] <- mod_ici
mod_list[[4]] <- mod_bf

## ------------------------------------------------------------------------ ##
##             	               Smoothed surfaces                            ##
## ------------------------------------------------------------------------ ##

## --- Local lik.

x11();
surface_plot(Mu_i.j_list[[1]], expression(paste(widehat(mu)[ij]," Local lik.")), c(min(Dur),max(Dur),min(Occ),max(Occ)))


## --- P-splines

x11();
surface_plot(Mu_i.j_list[[2]], expression(paste(widehat(mu)[ij]," P-splines")), c(min(Dur),max(Dur),min(Occ),max(Occ)))


## --- Local lik. ICI

x11();
surface_plot(Mu_i.j_list[[3]], expression(paste(widehat(mu)[ij]," Local lik. ICI")), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## --- Local lik. BF

x11();
surface_plot(Mu_i.j_list[[4]], expression(paste(widehat(mu)[ij]," Local lik. BF")), c(min(Dur),max(Dur),min(Occ),max(Occ)))

## ------------------------------------------------------------------------ ##
##             	       Comparisons :  forces of mortality                   ##
## ------------------------------------------------------------------------ ##

col.vec <- c(2,3,4,6, "orange")
lty.vec <- c(1,1,1,1,1)
leg.vec <- c(names.models,"Tech. report")

## ------------------ Graphical comparison for a particular age of occurence

## --- Choose an age of occurence

yy <- 70

x11();
comp.plot.occ(Mu_i.j_list, Mu_i.j, yy , c(135,140), col.vec, lty.vec, leg.vec, Dur, c(-0.01,0.15))

## ------------------ Graphical comparison for a particular duration

## --- Choose a duration (months)

xx <- 0

x11();
comp.plot.dur(Mu_i.j_list, Mu_i.j, xx , c(135,140), col.vec, lty.vec, leg.vec, Occ, c(-0.01,0.15))

## ------------------------------------------------------------------------ ##
##             	         Comparisons : number of deaths                     ##
## ------------------------------------------------------------------------ ##

D_i.j_list <- vector("list",5)

for (i in 1:5){ 
	D_i.j_list[[i]] <- Mu_i.j_list[[i]]*E_i.j 
	colnames(D_i.j_list[[i]]) <- as.character(Occ) }

## ------------------ Graphical comparison for a particular age of occurence

## --- Choose an age of occurence

yy <- 90

x11();
comp.plot.occ(D_i.j_list, D_i.j, yy , c(135,140), col.vec, lty.vec, leg.vec, Dur, c(-1,16))

## ------------------ Graphical comparison for a particular duration

## --- Choose a duration (months)

xx <- 0

x11();
comp.plot.dur(D_i.j_list, D_i.j, xx , c(135,140), col.vec, lty.vec, leg.vec, Occ, c(-1,70))

## ------------------------------------------------------------------------ ##
##             	                     Tests                                  ##
## ------------------------------------------------------------------------ ##

## ------------------ Standardized residuals

test.sdresid <- stand.resid(Mu_i.j_list, Mu_i.j, E_i.j, Dur, as.character(Occ), leg.vec)

test.sdresid$s.r.2
test.sdresid$s.r.3

## ------------------ Nber of runs & signs test:

test.runs <- runs_test(Mu_i.j_list, Mu_i.j, Dur, as.character(Occ), leg.vec)

test.runs

test.sign <- sign_test(Mu_i.j_list, Mu_i.j, Dur, as.character(Occ), leg.vec)

test.sign

## ------------------ chi^2, R2, MAPE

chi2R2MAPE <- chi2R2MAPE.test(Mu_i.j_list, D_i.j, E_i.j, Dur, as.character(Occ), leg.vec)

chi2R2MAPE

## ------------------ Deviance

load("Data/Smoothed data/dev_list.RData")

deviance <- vector(,5)
for (i in 1:5){ deviance[i] <- sum(dev_list[[i]])}

for (i in 1:5){ print(c(leg.vec[[i]], c(deviance[i]) )) }

## ------------------ Fitted Df

fdf <- vector(,4)
fdf[1] <- mod_lk$dp["df1"]
fdf[2] <- mod_ps$df
fdf[3] <- mod_ici$dp["df1"]
fdf[4] <- sum(mod_bf$InflMAT * mod_bf$lsd)

for (i in 1:4){ print(c(leg.vec[[i]], c(fdf[i]) )) }

