

## ------------------------------------------------------------------------ ##
##  R scripts for " Multidimenional smoothing by adaptive local kernel      ##
##                weighted log-likelihood: Application to long-term care    ##
##                insurance "                                               ##
## ------------------------------------------------------------------------ ##
##  Script        Sources                                                   ##
## ------------------------------------------------------------------------ ##
##  Description   Sources for misc and graphical functions                  ##
## ------------------------------------------------------------------------ ##
##  Author        Tomas Julien - 22-08-12                                   ##
## ------------------------------------------------------------------------ ##


## ------------------------------------------------------------------------ ##
##           	               Graphic functions                            ##
## ------------------------------------------------------------------------ ##

## ---------------- Surface plot

surface_plot = function(xx, zexpr, axis){
wireframe(xx, zlab=list(zexpr, rot=90), ylab=list("Age of occurence", rot=-46), xlab=list("Duration (months)"), aspect=c(2,1), drape = F, colorkey = F, screen = list(z = 20, x = -75), scales = list(arrows = F, y = list(at = c(seq(1,ncol(xx), by = 5)), labels = c(seq(axis[3],axis[4],by=5)), cex=.8), x = list( at = c(seq(1,121, by = 20)), labels = c(seq(axis[1],axis[2], by = 20)),cex=.8))) }

## ---------------- Plot residuals

plot.res.yy = function(obs.1,obs.2,obs.3, yy, xx, names1, names2) {
	yy <- as.character(yy)
	x1 <- x2 <- x3 <- matrix(,length(xx),length(names1))
	for (i in 1:5){
		x1[,i] <- obs.1[[i]][,yy]
		x2[,i] <- obs.2[[i]][,yy]
		x3[,i] <- obs.3[[i]][,yy] }
	MAT <- matrix(NA,length(xx)*length(names1),length(names2))
	MAT[,1] <- c(x1)
	MAT[,2] <- c(x2)
	MAT[,3] <- c(x3)
	VEC <- c(MAT)
	grp1 <- rep(1:length(names1), each = length(xx)*length(names2))
	grp2 <- rep(1:length(names2),times = length(names1), each = length(xx))
	dat <- as.data.frame(cbind(VEC, grp1, grp2))
	zzz <- with(dat, xyplot(VEC ~ rep(xx,15) | as.factor(grp2) + 
as.factor(grp1), data = dat, as.table = T, between=list(y = 0.6, x = 0.6),
type = "n", 
xlab = 'Duration (months)', ylab = "", layout=c(5,3),
par.settings = list(fontsize = list(text = 15, points = 7)),
par.strip.text = list(cex = .8, lines = 1.1),
scales = list(y = list(rot = 0, relation = "free", limits=c(list(c(range(x1[,1])),c(range(x2[,1])),c(range(x3[,1]))), list(c(range(x1[,2])),c(range(x2[,2])),c(range(x3[,2]))), list(c(range(x1[,3])),c(range(x2[,3])),c(range(x3[,3]))), list(c(range(x1[,4])),c(range(x2[,4])),c(range(x3[,4]))),list(c(range(x1[,5])),c(range(x2[,5])),c(range(x3[,5])))      )  ),
x = list(relation = "same", alternating = 1, tck = c(1,0), at = seq(min(xx),max(xx),20), labels = seq(min(xx),max(xx),20) )) ))
print(useOuterStrips(zzz, 
	strip.left = strip.custom(factor.levels = names1, horizontal = F, bg = "lightgrey"), 
	strip = strip.custom(factor.levels = names2, bg = "lightgrey"), 		strip.lines = 1.1, strip.left.lines = 1.1)) 
	for (i in 1:5){
	trellis.focus("panel", 1, i, highlight = FALSE)
	panel.abline(h = 0, lty = 2, col = "black", cex=.5)
	panel.points(xx, x1[,i], type="p", pch=16, col="black", cex=.5)
	fit.res <- locfit(x1[,i]  ~ xx, family = "gaussian", alpha = 0.15, link = "identity", kern="gauss")
	panel.points(xx, fitted(fit.res), type="l", col="black", cex=.5)
	trellis.unfocus()
	trellis.focus("panel", 2, i, highlight = FALSE)
	panel.abline(h = 2, lty = 2, col = "black", cex=.5)
	panel.abline(h = -2, lty = 2, col = "black", cex=.5)
	panel.abline(h = 3, lty = 3, col = "black", cex=.5)
	panel.abline(h = -3, lty = 3, col = "black", cex=.5)
	panel.points(xx, x2[,i], type="p", pch=16, col="black", cex=.5)
	trellis.unfocus()
	trellis.focus("panel", 3, i, highlight = FALSE)
        panel.points(xx, x3[,i], type = "b",col = "black", pch=16,cex=.5)
	trellis.unfocus() } }

## ---------------- Comparison plot for particular age of occurence

comp.plot.occ = function(x, obs, occ ,k, col.vec, lty.vec, leg.vec, age.vec, lly) {
	yy <- as.character(occ)
	expr <- paste("Comparison of the fits for the age of occurence: ", occ)
	xx <- matrix(,length(age.vec),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][age.vec+1-min(age.vec), yy]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(age.vec, length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Duration (months)',
	ylab = expression(widehat(mu)[uv]), type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(min(age.vec),max(age.vec)+1,10), labels = seq(min(age.vec),max(age.vec)+1,10)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(age.vec, obs[age.vec+1-min(age.vec), yy], type="p", col=1, pch=16)
	for( i in 1:length(x)) { panel.points(age.vec, x[[i]][age.vec+1-min(age.vec),yy], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",length(x))), pch=16, col = c(1,col.vec), lty = c(1,lty.vec)), cex = .9, text = list(lab = c("Observed", leg.vec)), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparison plot for particular duration

comp.plot.dur = function(x, obs, yy ,k, col.vec, lty.vec, leg.vec, occ, lly) {
	occ <- as.character(occ)
	expr <- paste("Comparison of the fits for the duration: ", yy)
	xx <- matrix(,length(occ),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][yy+1, occ]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(as.numeric(occ), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Age of occurence',
	ylab = expression(widehat(mu)[uv]), type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(70,90,2), labels = seq(70,90,2)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(as.numeric(occ), obs[yy+1, occ], type="p", col=1, pch=16)
	for( i in 1:length(x)) { panel.points(as.numeric(occ), x[[i]][yy+1, occ], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",length(x))), pch=16, col = c(1,col.vec), lty = c(1,lty.vec)), cex = .9, text = list(lab = c("Observed", leg.vec)), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparison survival function for particular age of occurrence

comp.plot.sf.occ = function(x, occ ,k, col.vec, lty.vec, leg.vec, age.vec, lly) {
	yy <- as.character(occ)
	expr <- paste("Comparison of the surv. fct. for the age of occurence: ", occ)
	xx <- matrix(,length(age.vec),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][age.vec+1-min(age.vec), yy]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(age.vec, length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Duration (months)',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(min(age.vec),max(age.vec)+1,10), labels = seq(min(age.vec),max(age.vec)+1,10)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(age.vec, x[[1]][age.vec+1-min(age.vec),yy], type="p", pch=16, col=col.vec[1], lty = lty.vec[1]) 
	for( i in 2:length(x)) { panel.points(age.vec, x[[i]][age.vec+1-min(age.vec),yy], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",length(x)-1)), pch=16, col = col.vec, lty = lty.vec), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparison life expectancy for particular age of occurrence

comp.plot.ple.occ = function(x, occ ,k, col.vec, lty.vec, leg.vec, age.vec, lly) {
	yy <- as.character(occ)
	expr <- paste("Period life exp. for the age of occurence: ", occ)
	xx <- matrix(,length(age.vec),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][age.vec+1-min(age.vec), yy]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(age.vec, length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Duration (months)',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(min(age.vec),max(age.vec)+1,10), labels = seq(min(age.vec),max(age.vec)+1,10)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(age.vec, x[[1]][age.vec+1-min(age.vec),yy], type="p", pch=16, col=col.vec[1], lty = lty.vec[1]) 
	for( i in 2:length(x)) { panel.points(age.vec, x[[i]][age.vec+1-min(age.vec),yy], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",length(x)-1)), pch=16, col = col.vec, lty = lty.vec), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparison survival function for particular duration

comp.plot.sf.dur = function(x, yy ,k, col.vec, lty.vec, leg.vec, occ, lly) {
	occ <- as.character(occ)
	expr <- paste("Comparison of the surv. fct. for the duration: ", yy)
	xx <- matrix(,length(occ),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][yy+1, occ]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(as.numeric(occ), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Age of occurence',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(70,90,2), labels = seq(70,90,2)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(as.numeric(occ), x[[1]][yy+1, occ], type="p", pch=16, col=col.vec[1], lty = lty.vec[1]) 
	for( i in 2:length(x)) { panel.points(as.numeric(occ), x[[i]][yy+1, occ], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",length(x)-1)), pch=16, col = col.vec, lty = lty.vec), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparison median month at death

comp.plot.med.month = function(xx, k, col.vec, lty.vec, leg.vec, occ, lly) {
	occ <- as.character(occ)
	expr <- paste("Comparison of the month median at death")
	nn <- length(c(xx[,1])); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx[,1]), grp1))
	zzz <- with(dat, xyplot(c(xx[,1]) ~ rep(as.numeric(occ), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Age of occurence',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(70,90,2), labels = seq(70,90,2)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(as.numeric(occ), xx[occ,1], type="p", pch=16, col=col.vec[1], lty = lty.vec[1]) 
	for( i in 2:ncol(xx)) { panel.points(as.numeric(occ), xx[occ,i], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",ncol(xx)-1)), col = col.vec, pch=16, lty = lty.vec), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparison standard dev of lifetime

comp.plot.sd.Fx = function(xx, k, col.vec, lty.vec, leg.vec, occ, lly) {
	occ <- as.character(occ)
	expr <- paste("Comparison of the std. deviation of the random lifetime")
	nn <- length(c(xx[,1])); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx[,1]), grp1))
	zzz <- with(dat, xyplot(c(xx[,1]) ~ rep(as.numeric(occ), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Age of occurence',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(70,90,2), labels = seq(70,90,2)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
		panel.points(as.numeric(occ), xx[occ,1], type="p", pch=16, col=col.vec[1], lty = lty.vec[1]) 
	for( i in 2:ncol(xx)) { panel.points(as.numeric(occ), xx[occ,i], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",ncol(xx)-1)), pch=16, col = col.vec, lty = lty.vec), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparison the entropy

comp.plot.H.Fx = function(xx, k, col.vec, lty.vec, leg.vec, occ, lly) {
	occ <- as.character(occ)
	expr <- paste("Comparison of the entropy")
	nn <- length(c(xx[,1])); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx[,1]), grp1))
	zzz <- with(dat, xyplot(c(xx[,1]) ~ rep(as.numeric(occ), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Age of occurence',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(70,90,2), labels = seq(70,90,2)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
	panel.points(as.numeric(occ), xx[occ,1], type="p", pch=16, col=col.vec[1], lty = lty.vec[1]) 
	for( i in 2:ncol(xx)) { panel.points(as.numeric(occ), xx[occ,i], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",ncol(xx)-1)), pch=16, col = col.vec, lty = lty.vec), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ---------------- Comparison life expectancy for particular duration

comp.plot.ple.dur = function(x, yy ,k, col.vec, lty.vec, leg.vec, occ, lly) {
	occ <- as.character(occ)
	expr <- paste("Period life exp. for the duration: ", yy)
	xx <- matrix(,length(occ),length(x))
	for (i in 1:length(x)) {
		xx[,i] <- x[[i]][yy+1, occ]
	}
	nn <- length(c(xx)); gr <- 1; grp1 <- as.factor(rep(gr, nn))
	dat <- as.data.frame(cbind(c(xx), grp1))
	zzz <- with(dat, xyplot(c(xx) ~ rep(as.numeric(occ), length(gr)) | as.factor(grp1), 
	data = dat, layout = c(1,1), as.table = T, xlab = 'Age of occurence',
	ylab = "", type= "n", scales = list(x = list(alternating = 1, tck = c(1,0),
	at = seq(70,90,2), labels = seq(70,90,2)), y = list(alternating = 3, 
	tck = c(1,1), limits= lly)), par.settings = list(fontsize = list(text = 15, points = 7)),
	between=list(y = 0.6, x = 0.6), par.strip.text = list(cex = 1, lines = 1.1),
	strip = strip.custom(which.given = 1, factor.levels = expr, bg = "lightgrey")))
	print(zzz)
	trellis.focus("panel", 1, 1, highlight = FALSE)
		panel.points(as.numeric(occ), x[[1]][yy+1, ], type="p", pch=16, col=col.vec[1], lty = lty.vec[1]) 
	for( i in 2:length(x)) { panel.points(as.numeric(occ), x[[i]][yy+1, ], type="l", col=col.vec[i], lty = lty.vec[i]) }
	trellis.unfocus()
k1 <- draw.key(list(lines = list( type = c("p",rep("l",length(x)-1)), pch=16, col = col.vec, lty = lty.vec), cex = .9, text = list(lab = leg.vec), draw=TRUE))

pushViewport(viewport(x = unit(k[1], "mm"), y = unit(k[2], "mm"), width = unit(1, "grobwidth", k1), height = unit(1, "grobheight", k1), just(c("left", "bottom"))))

grid.draw(k1) }

## ------------------------------------------------------------------------ ##
##           	          Local likelihood functions                        ##
## ------------------------------------------------------------------------ ##	

## --- Compute criteria and fitted DF

get.crit.lk = function(z, y, x, hh, pp, k, fam, lk, ww) { 
	LCVMAT <- AICMAT <- V2MAT <- span.mat <- nber.p.mat <-matrix(,length(hh),length(pp))
	for ( h in hh ) {
		for( p in (pp+1) ) { 
			nber.p <- get.enp.target(h)
			nber.p.mat[h, p] <- nber.p
			span <- nber.p / (length(x)*length(y))
			span.mat[h, p] <- span
			LCVMAT[h, p] <- lcv(c(as.matrix(z)) ~ lp(c(rep(x,length(y))),c(rep(y,each= length(x))),deg = p-1, nn = span,  scale=1), ev=dat(), family = fam, link = lk, kern = k, weights = ww)[4]
			AICMAT[h, p] <- aic(c(as.matrix(z)) ~ lp(c(rep(x,length(y))),c(rep(y,each= length(x))),deg = p-1, nn = span,  scale=1), ev=dat(), family = fam, link = lk, kern = k, weights = ww)[4]
			V2MAT[h, p] <-  locfit(c(as.matrix(z)) ~ lp(c(rep(x,length(y))),c(rep(y,each= length(x))),deg = p-1, nn = span,  scale=1), ev=dat(), family = fam, link = lk, kern = k, weights = ww)$dp["df2"] } } 
	return(list(LCVMAT = LCVMAT, AICMAT = AICMAT, V2MAT = V2MAT, span.mat = span.mat, nber.p.mat = nber.p.mat )) }

## --- Plot criterion

plot.crit = function(crit.name, crit, v2, lim.crit, lim.v2) { 
	figname <- paste(crit.name, "scores for various polynomial degrees")
	VECA <- c(crit); VECB <- c(v2)
	xyplot(VECA ~ VECB|as.factor(figname),xlab = "Fitted DF", ylab= crit.name, type="n", par.settings = list(fontsize = list(text = 15, points = 12)), par.strip.text = list(cex = 1, lines = 1.1), strip = strip.custom(bg = "lightgrey"), scales = list(y = list(relation = "free", limits = list(lim.crit), at = list(c(seq(min(lim.crit),max(lim.crit),by = (max(lim.crit)-min(lim.crit))/10)))), x = list(relation = "free", limits = lim.v2, tck = c(1,1))), panel = function(...) {
		llines(VECA[1:20]~VECB[1:20],type = "p", pch = "0", col = 1)
		llines(VECA[21:40]~VECB[21:40],type = "p", pch = "1", col = 1)
		llines(VECA[41:60]~VECB[41:60],type = "p", pch = "2", col = 1)
		llines(VECA[61:80]~VECB[61:80],type = "p", pch = "3", col = 1) } ) }

## --- Get the equivalent number of parameter
	
get.enp.target = function(h) {
	e.d.vec <- euc.dist(1:41, 1:41, 21, 21, c("mod.1", h))
	e.n.p.target <- length(which(e.d.vec <= h)==TRUE)
	return(e.n.p.target)
	}
	
## ---------------- This function returns the Euclidian distance if mod.1 is
## ---------------- specified. When mod.2 and mod.3 are specifed it scaled the
## ---------------- distance.

euc.dist = function(x1, x2, a1, a2, mod)
     {
      x1.vec <- rep(x1, times = length(x2))
      x2.vec <- rep(x2, each = length(x1))
      if (mod[1] == "mod.1") {
      VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec)^2)) }
      if (mod[1] == "mod.2") {
      h.1 <- ((as.numeric(mod[2]) * length(x1)) -1) / 2
	  h.2 <- ((as.numeric(mod[2]) * length(x2)) -1) / 2  
	  VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec) * (h.1 / h.2) )^2 ) }
	  if (mod[1] == "mod.3") {
      h.1 <- ((as.numeric(mod[2]) * length(x1)) -1) / 2
	  h.2 <- ((as.numeric(mod[3]) * length(x2)) -1) / 2
	  VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec) * (h.1 / h.2) )^2  ) }
	  if (mod[1] == "mod.4") {
      h.1 <- as.numeric(mod[2])
	  h.2 <- as.numeric(mod[3])
	  VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec) * (h.1 / h.2) )^2  ) }
	  if (mod[1] == "mod.5") {
      VEC <- sqrt((a1 - x1.vec)^2 + ((a2 - x2.vec)^2)) }
	 return(VEC)
     }

## ---------------- Define logit function

f.logit = function(x) { log(x/(1-x)) }

## ---------------- Bandwidth factors model
	
loc.lk.h.dep = function(s, n, x.mat, y.mat, h, p, k, s.p, offset)
	{
	n[n==0] <- 1
	f_i <-  (n/sum(n)) / min(n/sum(n)) 
	l_i <- (c(f_i))^-s.p
	h.dep <- round(h * l_i)
	#h.dep[h.dep < 6] <- 6
	#h.dep[h.dep > 20] <- 20
	h.to.do <- as.numeric(levels(as.factor(h.dep)))
	rslt.BetaMAT <- rslt.InflVEC <- rslt.SE <- rslt.lsd <- matrix(,length(c(x.mat)),length(h.to.do))
	for (i in 1:length(h.to.do)) {
		nber.p <- get.enp.target(h.to.do[i])
		span <- nber.p / (nrow(x.mat)*ncol(x.mat))
		FIT <- locfit(c(s) ~ c(x.mat)*c(y.mat),deg = p, alpha = span, family = "binmoial", link = "logit", kern = k,ev=dat(), scale=FALSE, weights = c(offset))
		fff <- predict(FIT , se.fit=T)
		rslt.BetaMAT[,i] <- matrix(fff$fit,length(Dur),length(Occ))
		rslt.InflVEC[,i] <- fitted(FIT, what="infl", where="fitp")
		rslt.lsd[,i] <- fitted(FIT, what="coef",type="lddot", ev="data")
		rslt.SE[,i] <- matrix(fff$se.fit,length(Dur),length(Occ))
	}
	final.BetaMAT <- final.InflVEC <- final.SE <- final.lsd <- vector(,length(c(x.mat)))
	ind <- factor(h.dep, labels=c(1:length(h.to.do)))
	for(j in 1:length(c(x.mat))) {
		final.BetaMAT[j] <- rslt.BetaMAT[j,ind[j]]
		final.InflVEC[j] <- rslt.InflVEC[j,ind[j]]
		final.SE[j] <- rslt.SE[j,ind[j]]
		final.lsd[j] <- rslt.lsd[j,ind[j]]
	}
	return(list(l_i = matrix(l_i,nrow(x.mat), ncol(x.mat)), 
				muxhat = matrix(final.BetaMAT,nrow(x.mat), ncol(x.mat)),
				h.dep = matrix(h.dep,nrow(x.mat), ncol(x.mat)),
				InflMAT = matrix(final.InflVEC,nrow(x.mat), ncol(x.mat)),
				se = matrix(final.SE,nrow(x.mat), ncol(x.mat)),
				lsd = matrix(final.lsd,nrow(x.mat), ncol(x.mat))
				))
	}

## ------------------------------------------------------------------------ ##
##           	                Tests functions                             ##
## ------------------------------------------------------------------------ ##	
## --- Standardized residuals

stand.resid = function(x, y, z, age.vec, year.vec, leg.name) {
	s.r <- vector("list",length(x))
	names(s.r) <- leg.name
	s.r.2 <- s.r.3 <- matrix(,2,length(x))
	colnames(s.r.2) <- colnames(s.r.3) <- leg.name
	rownames(s.r.2) <- rownames(s.r.3) <- c("False", "True")
	for(i in 1:length(x)){
	xx <- x[[i]][age.vec + 1 - min(age.vec),year.vec]
	yy <-  y[age.vec + 1- min(age.vec), year.vec]
	zz <- z[age.vec + 1- min(age.vec), year.vec]
	s.r[[i]] <- (xx - yy) / sqrt(xx/zz)
	colnames(s.r[[i]]) <- year.vec
	s.r.2[,i] <- table(abs(s.r[[i]])>2)
	s.r.3[,i] <- table(abs(s.r[[i]])>3) } 
	return(list(s.r = s.r, s.r.2 = s.r.2, s.r.3 = s.r.3)) }


### --- Runs test:

runs_test = function(x, y, age.vec, year.vec, leg.name){
rlt.all <- vector("list", length(x))
names(rlt.all ) <-  leg.name
rlt <- matrix(,5,length(x))
colnames(rlt) <- leg.name
rownames(rlt) <- c("Nber of runs", "Signs (-)", "Signs (+)", "Statistic", "p.value")
for (i in 1:length(x)) {
yy <- sign((y)[age.vec-min(age.vec)+1, year.vec ] -  x[[i]][age.vec + 1 - min(age.vec) , year.vec])
fac.SIGN <- factor(as.matrix(val.SIGN))
sign.neg <- sum(levels(fac.SIGN)[1] == fac.SIGN)
sign.pos <- sum(levels(fac.SIGN)[2] == fac.SIGN)
val.RUNS <- 1 + sum(as.numeric(fac.SIGN[-1] != fac.SIGN[-length(fac.SIGN)]))
mean.run <- 1 + 2*sign.neg*sign.pos / (sign.neg+sign.pos)
var.run <- 2*sign.neg*sign.pos * (2*sign.neg*sign.pos - sign.neg - sign.pos) / ((sign.neg+sign.pos)^2 * (sign.neg+sign.pos-1))
xi.RUNS <- (val.RUNS - mean.run) / sqrt(var.run)
p.val.RUNS <- 2*(1-pnorm(abs(xi.RUNS)))
rlt[1,i] <- val.RUNS
rlt[2,i] <- sign.neg; rlt[3,i] <- sign.pos; 
rlt[4,i] <- xi.RUNS; rlt[5,i] <- p.val.RUNS }
return(rlt = rlt) }

sign_test = function(x, y, age.vec, year.vec, leg.name){
rlt.all <- vector("list", length(x))
names(rlt.all ) <-  leg.name
rlt <- matrix(,5,length(x))
colnames(rlt) <- leg.name
rownames(rlt) <- c("Signs (-)", "Signs (+)", "Statistic", "p.value")
for (i in 1:length(x)) {
yy <- sign((y)[age.vec-min(age.vec)+1, year.vec ] -  x[[i]][age.vec + 1 - min(age.vec) , year.vec])
fac.SIGN <- factor(as.matrix(val.SIGN))
sign.neg <- sum(levels(fac.SIGN)[1] == fac.SIGN)
sign.pos <- sum(levels(fac.SIGN)[2] == fac.SIGN)
xi.SIGN <- (abs(sign.pos-sign.neg)-1) / sqrt(sign.pos+sign.neg)
p.val.SIGN <- 2*(1-pnorm(abs(xi.SIGN)))
rlt[1,i] <- sign.neg; rlt[2,i] <- sign.pos; 
rlt[3,i] <- xi.SIGN; rlt[4,i] <- p.val.SIGN }
return(rlt = rlt) }

### --- Kolmogorov Smirnov test:

kstest = function(x, y, age.vec, year.vec, leg.name){
	rlt <- matrix(,2,length(x))
	colnames(rlt) <- leg.name
	rownames(rlt) <- c("Statistic", "p.value")
	for (i in 1:length(x)) {
		xx <- x[[i]][age.vec + 1 - min(age.vec), as.character(year.vec)]
		yy <-  y[age.vec + 1 - min(age.vec), as.character(year.vec)]
		kk <- ks.test(yy, xx)
		rlt[1,i] <- kk$statistic; rlt[2,i] <- kk$p.value 	}
		return(rlt)
}

### --- chi^2, R2, MAPE

chi2R2MAPE.test = function(x, y, z, age.vec, year.vec, leg.name){
	rlt <- matrix(,3,length(x))
	colnames(rlt) <- leg.name
	rownames(rlt) <- c("Chi^2", "R2", "MAPE")
	for (i in 1:length(x)) {
		xx <- x[[i]][age.vec + 1 - min(age.vec), as.character(year.vec)]
		yy <-  y[age.vec + 1- min(age.vec), as.character(year.vec)]
		zz <- z[age.vec + 1- min(age.vec), as.character(year.vec)]
		chi2 <- sum(((yy - zz * xx)^2) / (zz * xx * (1-xx)), na.rm=T)
		R2 <- 1 - (sum(((yy/zz - xx)^2)[yy>0], na.rm=T) / sum((
		(yy/zz - mean((yy/zz)[yy>0]))^2), na.rm=T))
		MAPE <- sum(abs((yy/zz - xx)/(yy/zz))[yy>0])/sum(yy>0) * 100
		rlt[1,i] <- chi2; rlt[2,i] <- R2; rlt[3,i] <- MAPE 	}
		return(rlt)
}

## ------------------------------------------------------------------------ ##
##          	          Survival index and markers                        ##
## ------------------------------------------------------------------------ ##

survi.fct = function(qx.p,l_0,x) {
	lx.p <- vector(,length(x))
	lx.p[1] <- l_0
	for (i in 1:length(x)) {
	lx.p[1+i] <- (1-qx.p[i])*lx.p[i] }
	S.x <- lx.p / l_0
	return(S.x = S.x[1:length(x)])
	}

## --- ple function plot

period.l.e = function(mu,age,year){
	ple <- matrix(,length(age),length(year))
	colnames(ple) <- as.character(year)
	for(y in 1:length(year)){
		for(a in 1:length(age)){
			p1 <- cumprod(exp(-mu[(age[a]-min(age)+1):(max(age)-min(age)+1), as.character(year[y])]))
			ple[a,y] <- sum(p1) } } 
			return(ple = ple) }

get.s.xx <- function(s,a) {
	x.xx <- (0:99)/100
	nrs <- nrow(s); ncs <- ncol(s)
	a.x <- a[1] : a[2]
	s.xx <- matrix(,100*(length(a.x)-1),nrs)
	a.xx <- vector(,100*(length(a.x)-1))
	for (i in 1:nrs) {
		l <- 0
		for (j in 1:(length(a.x)-1)) {
			for (k in 1:100) {
				s.xx[l+k,i] <- (1-x.xx[k])*s[i,j]+x.xx[k]*s[i,j+1] 
				a.xx[l+k] <- a.x[j]+x.xx[k] } 
			l <- l + 100 }
			}
	return(list(s.xx = s.xx, a.xx = a.xx)) }


